"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rsup_duration_1 = require("rsup-duration");
var react_1 = require("react");
var Waiter = /** @class */ (function () {
    function Waiter() {
        this._pending = Object.create(null);
        this._listeners = Object.create(null);
        this.useWait = this.useWait.bind(this);
    }
    Waiter.prototype.isPending = function (order) {
        return order in this._pending;
    };
    Waiter.prototype.promise = function (order, promise) {
        var _this = this;
        if (typeof order !== 'string') {
            throw new TypeError("Expected \"order\" to be of type \"string\", but \"" + typeof order + "\".");
        }
        if (typeof promise === 'function')
            promise = promise();
        if (!promise || typeof promise.then !== 'function') {
            throw new TypeError("Expected \"promise\" to be thenable.");
        }
        if (this.isPending(order)) {
            this._pending[order]++;
        }
        else {
            this._pending[order] = 1;
            this._emit(order);
        }
        var onFinally = function () {
            if (--_this._pending[order] > 0)
                return;
            delete _this._pending[order];
            _this._emit(order);
        };
        promise.then(onFinally, onFinally);
        return promise;
    };
    Waiter.prototype._emit = function (order) {
        (this._listeners[order] || []).forEach(function (fn) { return fn(); });
    };
    Waiter.prototype._subscribe = function (order, listener) {
        var _this = this;
        if (!(order in this._listeners))
            this._listeners[order] = [];
        var listeners = this._listeners[order];
        listeners.push(listener);
        return function () {
            listeners.splice(listeners.indexOf(listener), 1);
            if (listeners.length === 0)
                delete _this._listeners[order];
        };
    };
    // tslint:disable-next-line: cognitive-complexity
    Waiter.prototype.useWait = function (order, _a) {
        var _this = this;
        var _b = _a === void 0 ? {} : _a, _c = _b.delay, delay = _c === void 0 ? 0 : _c, _d = _b.duration, duration = _d === void 0 ? 0 : _d;
        var _e = react_1.useState(this.isPending(order)), isWaiting = _e[0], setWaiting = _e[1];
        var prevRef = react_1.useRef(isWaiting);
        react_1.useLayoutEffect(function () { prevRef.current = isWaiting; }, [isWaiting]);
        react_1.useLayoutEffect(function () {
            var delayer = delay > 0 ? rsup_duration_1.default(delay) : null;
            var persister = duration > 0 ? rsup_duration_1.default(duration) : null;
            var next = null;
            var unmounted = false;
            if (isWaiting && persister) {
                persister.start().then(endWaiting);
            }
            var listener = function () {
                var curr = _this.isPending(order);
                var prev = prevRef.current;
                if ((delayer && delayer.isDuring) || (persister && persister.isDuring)) {
                    next = curr;
                    return;
                }
                if (curr === prev)
                    return;
                if (curr) {
                    if (delayer) {
                        delayer.start().then(startWaiting);
                    }
                    else {
                        startWaiting();
                    }
                }
                else {
                    setWaiting(false);
                }
            };
            var off = _this._subscribe(order, listener);
            return function () {
                unmounted = true;
                off();
            };
            function startWaiting() {
                if (unmounted)
                    return;
                if (next !== false) {
                    setWaiting(true);
                    if (persister)
                        persister.start().then(endWaiting);
                }
                next = null;
            }
            function endWaiting() {
                if (unmounted)
                    return;
                if (next === false)
                    setWaiting(false);
                next = null;
            }
        }, [delay, duration]);
        return isWaiting;
    };
    return Waiter;
}());
exports.Waiter = Waiter;
exports.createWaiter = function () { return new Waiter(); };
exports.default = exports.createWaiter;
